﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Forms;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Drawing;
using System.Threading;
using System.Windows.Threading;

namespace program
{
    class GlavniProzor : Window
    {
        // inicijalizacija prozora

        private Container cont = new Container();

        public GlavniProzor()
        {
            this.WindowStartupLocation = WindowStartupLocation.CenterScreen;
            this.Title = "Stochastic Diffusion Search";
            this.Height = 720;
            this.Width = 1080;
            this.Background = new LinearGradientBrush(Colors.RoyalBlue, Colors.Gold, 15);
            this.AddChild(cont);
        }
    }                                          

    class Ulaz
    {
        // pokretanje aplikacije

        [STAThread]
        static void Main()
        {
            System.Windows.Application app = new System.Windows.Application();
            app.Run(new GlavniProzor());
        }
    }                                                           

    class Skroler : ScrollViewer
    {
        // kontrola za prikazivanja teksta

        private Ispis _ispis = new Ispis();

        public Skroler()
        {
            this.AddChild(_ispis);
            this.Margin = new Thickness(20);
        }

        public void _dodaj_tekst(string txt)
        {
            _ispis.Text += txt + "\n";
            ScrollToVerticalOffset(100);
        }

        public string _uzmi_tekst()
        {
            return _ispis.Text;
        }

        public void _ocisti()
        {
            _ispis.Text = null;
        }
    }                                         

    class Gumb : System.Windows.Controls.Button
    {
        // gumbi

        public Gumb(string natpis)
        {
            this.Height = 30;
            this.Width = 100;
            this.Content = natpis;
        }
    }

    class Ispis : TextBlock
    {
        // kontrola za prikazivanja teksta

        public Ispis()
        {
            this.Background = System.Windows.Media.Brushes.Black;
            this.Foreground = System.Windows.Media.Brushes.White;
            this.FontFamily = new System.Windows.Media.FontFamily("courier new");
            this.FontSize = 12;
            this.Text = "Mjesto za ispisivanje međurezultata";
        }
    }                                        

    class Rub : Border
    {
        // rubovi

        public Rub()
        {
            this.BorderThickness = new Thickness(2);
            this.CornerRadius = new CornerRadius(15);
            this.BorderBrush = System.Windows.Media.Brushes.White;
            this.Margin = new Thickness(20);
        }
    }

    class MjestoZaSliku : Canvas
    {
        // ovdje postavljamo slike na površinu prozora

        private string _adresa;
        private bool _ucitano;

        public MjestoZaSliku()
        {
            this.Margin = new Thickness(30);
        }

        public void _Ucitaj(string adresa)
        {
            ImageBrush fila = new ImageBrush();
            fila.ImageSource = new BitmapImage(new Uri(adresa, UriKind.Relative));
            this.Background = fila;
            _ucitano = true;
            _adresa = adresa;
        }

        public string _Daj_adresu()
        {
            return _adresa;
        }

        public bool _Je_ucitano()
        {
            if (_ucitano)
                return true;

            else
                return false;
        }
    }

    class Container : Grid
    {
        // glavna klasa u kojoj se postavljaju svi elementi
        // i izvršavaju metode vezane za rad aplikacije

        private Rub _mali_rub = new Rub();
        private Rub _veliki_rub = new Rub();
        private Skroler _skroler = new Skroler();
        private Gumb _velika_gmb = new Gumb("Učitaj veliku sliku");
        private Gumb _mala_gmb = new Gumb("Učitaj malu sliku");
        private Gumb _pokreni_gmb = new Gumb("Pokreni traženje");
        private Gumb _sejv_gmb = new Gumb("Spremi ispis");
        private MjestoZaSliku _mjesto_velika = new MjestoZaSliku();
        private MjestoZaSliku _mjesto_mala = new MjestoZaSliku();

        public Container()
        {
            this.Children.Add(_mali_rub);
            this.Children.Add(_veliki_rub);
            this.Children.Add(_skroler);
            this.Children.Add(_velika_gmb);
            this.Children.Add(_mala_gmb);
            this.Children.Add(_pokreni_gmb);
            this.Children.Add(_sejv_gmb);
            this.Children.Add(_mjesto_velika);
            this.Children.Add(_mjesto_mala);

            this.RowDefinitions.Add(new RowDefinition());
            this.ColumnDefinitions.Add(new ColumnDefinition());
            this.RowDefinitions.Add(new RowDefinition());
            this.ColumnDefinitions.Add(new ColumnDefinition());
            this.RowDefinitions.Add(new RowDefinition());
            this.ColumnDefinitions.Add(new ColumnDefinition());
            this.RowDefinitions.Add(new RowDefinition());
            this.ColumnDefinitions.Add(new ColumnDefinition());
            this.RowDefinitions.Add(new RowDefinition());
            this.ColumnDefinitions.Add(new ColumnDefinition());
            this.RowDefinitions.Add(new RowDefinition());
            this.ColumnDefinitions.Add(new ColumnDefinition());
            this.RowDefinitions.Add(new RowDefinition());
            this.ColumnDefinitions.Add(new ColumnDefinition());

            Grid.SetColumn(_skroler, 4);
            Grid.SetColumnSpan(_skroler, 3);
            Grid.SetRowSpan(_skroler, 5);

            Grid.SetColumn(_velika_gmb, 5);
            Grid.SetRow(_velika_gmb, 5);

            Grid.SetColumn(_mala_gmb, 5);
            Grid.SetRow(_mala_gmb, 6);

            Grid.SetColumn(_sejv_gmb, 6);
            Grid.SetRow(_sejv_gmb, 6);

            Grid.SetColumn(_pokreni_gmb, 6);
            Grid.SetRow(_pokreni_gmb, 5);

            Grid.SetRowSpan(_mjesto_velika, 5);
            Grid.SetColumnSpan(_mjesto_velika, 4);

            Grid.SetRowSpan(_mjesto_mala, 2);
            Grid.SetColumnSpan(_mjesto_mala, 2);
            Grid.SetColumn(_mjesto_mala, 1);
            Grid.SetRow(_mjesto_mala, 5);

            Grid.SetRowSpan(_veliki_rub, 5);
            Grid.SetColumnSpan(_veliki_rub, 4);

            Grid.SetRowSpan(_mali_rub, 2);
            Grid.SetColumnSpan(_mali_rub, 2);
            Grid.SetColumn(_mali_rub, 1);
            Grid.SetRow(_mali_rub, 5);

            _mala_gmb.Click += ucitaj_malu;
            _velika_gmb.Click += ucitaj_veliku;
            _pokreni_gmb.Click += start;
            _sejv_gmb.Click += spremi;
        }

        private void ucitaj_malu(object s, EventArgs e)
        {
            OpenFileDialog dijalog = new OpenFileDialog();
            dijalog.Title = "Učitaj malu bitmapu";

            if (dijalog.ShowDialog() == DialogResult.OK)
            {
                _mjesto_mala._Ucitaj(dijalog.FileName);
            }
        }

        private void ucitaj_veliku(object s, EventArgs e)
        {
            OpenFileDialog dijalog = new OpenFileDialog();
            dijalog.Title = "Učitaj veliku bitmapu";

            if (dijalog.ShowDialog() == DialogResult.OK)
            {
                _mjesto_velika._Ucitaj(dijalog.FileName);
            }
        }

        private void start(object s, EventArgs e)
        {
            if (_mjesto_mala._Je_ucitano() && _mjesto_velika._Je_ucitano())
            {
                _skroler._ocisti();
                POSAO(_mjesto_mala._Daj_adresu(), _mjesto_velika._Daj_adresu());
            }

            else
                System.Windows.MessageBox.Show("Nisu učitane obje datoteke", "Greška", MessageBoxButton.OK, MessageBoxImage.Stop);
        }

        private void spremi(object s, EventArgs e)
        {
            SaveFileDialog dijalog = new SaveFileDialog();
            dijalog.Filter = "Txt datoteka | *.txt";
            dijalog.Title = "Spremi ispis";

            if (dijalog.ShowDialog() == DialogResult.OK)
            {
                File.WriteAllText(dijalog.FileName, _skroler._uzmi_tekst());
            }
        }

        private koordinate rezultat;
        
        public delegate void skrolerDelegate(string tekst);

        private static Semaphore _pool;
        private static Semaphore _p;
        private static Semaphore _m;
        private static Semaphore _n;

        public static Boolean znakkraja = false; // varijabla kojom se sve dretve zaustave, dretva koja nadje rezultat će znakkraja postaviti na true

        private void POSAO(string adr_mala, string adr_velika)
        {          
            Bitmap mala = new Bitmap(adr_mala);
            Bitmap velika = new Bitmap(adr_velika);
         
            _pool = new Semaphore(0, 4);
            _p = new Semaphore(0, 4);
            _m = new Semaphore(0, 4);
            _n = new Semaphore(0, 4);
             
            rezultat = new koordinate(-1,-1);
            
            _skroler._dodaj_tekst("Inicijalizacija agenata (dretve pretrazivaci).");

            Thread dretva1 = new Thread(new ParameterizedThreadStart(trazilica));
            Thread dretva2 = new Thread(new ParameterizedThreadStart(trazilica));
            Thread dretva3 = new Thread(new ParameterizedThreadStart(trazilica));
            Thread dretva4 = new Thread(new ParameterizedThreadStart(trazilica));

            

            dretva1.Start(new trazilica_params(mala, velika, 0, 0, (velika.Width / 2) - 1 , (velika.Height / 2)-1));
            dretva2.Start(new trazilica_params(mala, velika, (velika.Width / 2)  ,0,velika.Width-1, (velika.Height / 2)-1 ));
            dretva3.Start(new trazilica_params(mala, velika, 0, (velika.Height / 2), (velika.Width /2)-1, velika.Height-1 ));
            dretva4.Start(new trazilica_params(mala, velika, (velika.Width /2),(velika.Height/2 ), velika.Width-1 , velika.Height-1 ));

            Thread.Sleep(500);
            _skroler._dodaj_tekst("Rezultat početni, rješenje još ne postoji : x=" + rezultat.x.ToString() + ", y=" + rezultat.y.ToString());
            _pool.Release(4);
            _p.Release(4);
            _m.Release(4);
            _n.Release(4);
            

        }


        public void trazilica(object data)
        {
            trazilica_params parametri = (trazilica_params)data;
            Bitmap mala = (Bitmap) parametri.mala.Clone();
            Bitmap velika = (Bitmap) parametri.velika.Clone();
            int x1 = parametri.x1;
            int y1 = parametri.y1;
            int x2 = parametri.x2;
            int y2 = parametri.y2;

            int i, j; //varijable petje, koordinate velike slike
            int k, l; //koordinate male
            int w = velika.Width - mala.Width;
            int h = velika.Height - mala.Height;
            bool zastavica;

            Thread.Sleep (500); 

            skrolerDelegate metodaSkrolerDelegate = new skrolerDelegate(_skroler._dodaj_tekst);

            Dispatcher.BeginInvoke(DispatcherPriority.Normal, metodaSkrolerDelegate, "Agent je aktiviran.");

            int bglb = mala.GetPixel(0, 0).ToArgb(); //malaavrijednostboje(mala, 0, 0);;// boja gornjeg lijevog bita
            
            for (i = x1; i <= x2 && i <= w; i++)
            {
                
                for (j = y1; j <= y2 && j <= h; j++)
                {

                    _n.WaitOne();
                    if ((j % 10 == 0) && ( i % 10 == 0))
                    {
                       Dispatcher.BeginInvoke(DispatcherPriority.Normal, metodaSkrolerDelegate, "Trenutna pozicija je : x=" + i.ToString() + ", y=" + j.ToString());
                    }
                    _n.Release();
                    if  (znakkraja == true) return;
                    _m.WaitOne();
                    if (bglb == velika.GetPixel(i, j).ToArgb())   // 
                    {
                        zastavica = true;
                        for (k = 0; k < mala.Width; k++)
                        {
                            for (l = 0; l < mala.Height; l++)
                            {
                                
                                if (velika.GetPixel(i + k, j + l).ToArgb() != mala.GetPixel(k, l).ToArgb())
                                {
                                    zastavica = false;
                                    break; 
                                }
                                
                            }
                        }
                        
                        _p.WaitOne();
                        if (zastavica == true) {

                                    znakkraja = true;
                                    rezultat = new koordinate(i, j);
                                    System.Windows.Forms.MessageBox.Show("Pretraga je gotova.");
                                    Dispatcher.BeginInvoke(DispatcherPriority.Normal, metodaSkrolerDelegate, "Rezultat agenta  je : x=" + rezultat.x.ToString() + ", y=" + rezultat.y.ToString());
                                

                            return;
                        }
                        _p.Release();                        
                    }
                    _m.Release();
                                        
                }
            }
        }

        public class koordinate
        {
            public int x;
            public int y;
            public koordinate(int _x, int _y)
            {
                this.x = _x;
                this.y = _y;
            }
        }


        public class trazilica_params
        {
            public Bitmap mala;
            public Bitmap velika;
            public int x1;
            public int y1;
            public int x2;
            public int y2;

            public trazilica_params(Bitmap _mala, Bitmap _velika, int _x1, int _y1, int _x2, int _y2)
            {
                this.mala = _mala;
                this.velika = _velika;
                this.x1 = _x1;
                this.y1 = _y1;
                this.x2 = _x2;
                this.y2 = _y2;
            }

        }  
    }

}